/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved. 
 *
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

package com.ibm.team.calm.foundation.rcp.ui.internal.preview;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;

import com.ibm.team.calm.foundation.client.preview.IResourceReference;
import com.ibm.team.calm.foundation.client.preview.ResourceReferenceResolver;
import com.ibm.team.calm.foundation.common.preview.IResourcePreviewManager;
import com.ibm.team.calm.foundation.common.preview.ResourcePreview;
import com.ibm.team.foundation.common.internal.util.XMLBuilder;
import com.ibm.team.jface.preview.DomainAdapter;
import com.ibm.team.jface.preview.DomainAdapterUtils;
import com.ibm.team.jface.preview.IDomainAdapter;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.util.NLS;
import com.ibm.team.repository.transport.client.AuthenticationException;

import java.net.URI;
import java.util.HashMap;

/**
 * Provides small and rich hover for instances of <code>IResourceReference</code>.
 * 
 * @see DomainAdapter
 * @since 3.0
 */
public class ResourceReferenceDomainAdapter extends DomainAdapter {

	/*
	 * @see com.ibm.team.workitem.ide.ui.internal.preview.WorkItemDomainAdapter#generateContentAsText(java.lang.Object)
	 */
	public String generateContentAsText(Object input) {

		/* Resource Reference */
		if (input instanceof IResourceReference) {
			Object resolved= ((IResourceReference) input).resolve();

			/* Compact Rendering */
			if (resolved instanceof ResourcePreview) {
				ResourcePreview ref= (ResourcePreview) resolved;
				if (ref.getTitle() != null)
					return ref.getTitle().getPlainText();
			}

			/* Anything Else - Delegate */
			else if (resolved != null) {
				IDomainAdapter adapter= DomainAdapterUtils.getDomainAdapter(resolved);
				if (adapter != null)
					return adapter.generateContentAsText(resolved);
			}
		}

		return ""; //$NON-NLS-1$
	}

	/*
	 * @see com.ibm.team.workitem.ide.ui.internal.preview.WorkItemDomainAdapter#generateContentAsHTML(java.util.HashMap, java.lang.StringBuffer, java.lang.Object, com.ibm.team.jface.preview.IDomainAdapter.Info)
	 */
	public void generateContentAsHTML(HashMap head, StringBuffer body, Object input, Info info) {
		generateContentAsHTML(head, body, input, info, new NullProgressMonitor());
	}

	/*
	 * @see com.ibm.team.workitem.ide.ui.internal.preview.WorkItemDomainAdapter#generateContentAsHTML(java.util.HashMap, java.lang.StringBuffer, java.lang.Object, com.ibm.team.jface.preview.IDomainAdapter.Info, org.eclipse.core.runtime.IProgressMonitor)
	 */
	@SuppressWarnings("restriction")
	public void generateContentAsHTML(HashMap head, StringBuffer body, Object input, Info info, IProgressMonitor monitor) {

		/* Resource Reference */
		if (input instanceof IResourceReference) {
			IResourceReference reference= (IResourceReference) input;
			Object resolved= ((IResourceReference) input).resolve();

			/* Start Resolving if not yet done or if we need to authenticate */
			if (resolved == null || (resolved != null && resolved instanceof ResourcePreview && ((ResourcePreview)resolved).hasAuthenticationError())) {

				/* Create Resolver */
				ResourceReferenceResolver resolver= new ResourceReferenceResolver();

				/* Hover Tooltip */
				if (info.isHoverTooltip) {
					try {
						resolver.resolve(reference, IResourcePreviewManager.DEFAULT, false, new NullProgressMonitor()); //Need this because caller (Hover) cancels monitor on close
						resolved= reference.resolve();
					}

					/* Error: Ask user to log in */
					catch (AuthenticationException e) {
						XMLBuilder builder= new XMLBuilder();

						if (reference.getURI() != null && reference.getURI().isAbsolute())
							builder.xml("<p>").plain(NLS.bind(Messages.ResourceReferenceDomainAdapter_F2_LOGIN_HOST, reference.getURI().getScheme() + "://" + reference.getURI().getHost())).xml("</p>"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
						else
							builder.xml("<p>").plain(Messages.ResourceReferenceDomainAdapter_F2_LOGIN).xml("</p>"); //$NON-NLS-1$ //$NON-NLS-2$
						body.append(builder);

						return;
					} catch (TeamRepositoryException ignored) {
					} // falls through
				}

				/* Focus Tooltip */
				else {
					try {
						resolver.resolve(reference, IResourcePreviewManager.DEFAULT, true, new NullProgressMonitor()); //Need this because caller (Hover) cancels monitor on close
						resolved= reference.resolve();
					} catch (AuthenticationException e) {
						body.append(e.getMessage());
					} catch (TeamRepositoryException ignored) {
					} // falls through
				}
			}

			/* We resolved something */
			if (resolved != null 
					&& (!(resolved instanceof ResourcePreview)  
							|| resolved instanceof ResourcePreview && !((ResourcePreview)resolved).hasErrors())) {

				/* Compact Rendering */
				if (resolved instanceof ResourcePreview) {
					ResourcePreview ref= (ResourcePreview) resolved;
					
					/* Fallback to show URI if URIs are not provided */
					if (ref.getSmallPreviewURI() == null && ref.getLargePreviewURI() == null) {
						showTooltipForUnknownResource(body, reference, info);
					}

					/* Hover Tooltip */
					else if (info.isHoverTooltip) {
						URI smallPreviewURI= ref.getSmallPreviewURI() != null ? ref.getSmallPreviewURI() : ref.getLargePreviewURI();
						String smallPreviewURIString= smallPreviewURI.toString();
						
						Long desiredWidth= ref.getSmallPreviewWidth() != null ? ref.getSmallPreviewWidth() : ref.getLargePreviewWidth();
						Long desiredHeight= ref.getSmallPreviewHeight() != null ? ref.getSmallPreviewHeight() : ref.getLargePreviewHeight();

						/* Workaround until we have a compact hover in the compact rendering spec */
						boolean hasQuery= (smallPreviewURI.getQuery() != null);
						smallPreviewURIString+= hasQuery ? "&_compactHover=true" : "?_compactHover=true"; //$NON-NLS-1$ //$NON-NLS-2$

						loadCompactRendering(head, body, ref, URI.create(smallPreviewURIString), desiredWidth, desiredHeight, info);
					}
				
					/* Focus Tooltip */
					else {
						URI largePreviewURI= ref.getLargePreviewURI() != null ? ref.getLargePreviewURI() : ref.getSmallPreviewURI();
						Long desiredWidth= ref.getLargePreviewWidth() != null ? ref.getLargePreviewWidth() : ref.getSmallPreviewWidth();
						Long desiredHeight= ref.getLargePreviewHeight() != null ? ref.getLargePreviewHeight() : ref.getSmallPreviewHeight();
						
						loadCompactRendering(head, body, ref, largePreviewURI, desiredWidth, desiredHeight, info);
					}
				} 
				
				/* Anything Else - Try to delegate */
				else {
					IDomainAdapter adapter= DomainAdapterUtils.getDomainAdapter(resolved);
					if (adapter instanceof DomainAdapter) {
						((DomainAdapter) adapter).generateContentAsHTML(head, body, resolved, info, monitor);
					} else if (adapter != null) {
						adapter.generateContentAsHTML(head, body, input, info);
					} else {
						showTooltipForUnknownResource(body, reference, info);
					}
				}
			}

			/* Fallback to show URI */
			else {
				showTooltipForUnknownResource(body, reference, info);
			}
		}
	}

	@SuppressWarnings("restriction")
	private void showTooltipForUnknownResource(StringBuffer body, IResourceReference reference, Info info) {
		XMLBuilder builder= new XMLBuilder();

		String comment= reference.getComment();
		String uri= reference.getURI() != null ? reference.getURI().toString() : null;

		/* Hover Tooltip */
		if (info.isHoverTooltip) {
			if (comment != null && comment.length() > 0)
				builder.plain(comment);
			else if (uri != null && uri.length() > 0)
				builder.plain(uri);
		}

		/* Focus Tooltip */
		else {
			if (uri != null && uri.length() > 0 && comment != null && comment.length() > 0)
				builder.xml("<a href=\"").plain(uri).xml("\">").plain(comment).xml("</a>"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			else if (uri != null && uri.length() > 0)
				builder.xml("<a href=\"").plain(uri).xml("\">").plain(uri).xml("</a>"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			else if (comment != null && comment.length() > 0)
				builder.plain(comment);
		}

		body.append(builder);
	}

	private void loadCompactRendering(HashMap head, StringBuffer body, ResourcePreview ref, URI previewUri, Long desiredWidth, Long desiredHeight, Info info) {
		if (true)
			loadCompactRenderingWithIFrame(body, previewUri, desiredWidth, desiredHeight, info);
		else
			loadCompactRenderingWithMetaRefresh(head, body, ref, previewUri, info);
	}

	private void loadCompactRenderingWithIFrame(StringBuffer body, URI previewUri, Long desiredWidth, Long desiredHeight, Info info) {
		String desiredWidthVal= (desiredWidth != null) ? desiredWidth.toString() : "100%"; //$NON-NLS-1$ 
		String desiredHeightVal= (desiredHeight != null) ? desiredHeight.toString() : "100%"; //$NON-NLS-1$ 

		XMLBuilder iFrame= new XMLBuilder();
		iFrame.xml("<iframe frameborder=\"0\" allowtransparency=\"true\" width=\"").plain(desiredWidthVal).xml("\" height=\"").plain(desiredHeightVal).xml("\" src=\""); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		iFrame.plain(previewUri.toString()).plain(ResourceReferenceHyperlinkHandler.URL_MARKER).xml("\"></iframe>"); //$NON-NLS-1$

		body.append(iFrame);
	}

	private void loadCompactRenderingWithMetaRefresh(HashMap head, StringBuffer body, ResourcePreview ref, URI previewUri, Info info) {
		StringBuilder meta= new StringBuilder();
		meta.append("<meta HTTP-EQUIV=\"REFRESH\" content=\"1; url="); //$NON-NLS-1$
		meta.append(previewUri);
		meta.append(ResourceReferenceHyperlinkHandler.URL_MARKER);
		meta.append("\">"); //$NON-NLS-1$
		head.put(previewUri, meta.toString());

		Long desiredWidth= info.isHoverTooltip ? ref.getSmallPreviewWidth() : ref.getLargePreviewWidth();
		Long desiredHeight= info.isHoverTooltip ? ref.getSmallPreviewHeight() : ref.getLargePreviewHeight();

		body.append("<div style=\"width: ").append(desiredWidth).append("px; height: ").append(desiredHeight).append("px;\">&nbsp;</div>"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}

	/*
	 * @see com.ibm.team.workitem.ide.ui.internal.preview.WorkItemDomainAdapter#generateTitle(java.lang.Object)
	 */
	public String generateTitle(Object input) {

		/* Resource Reference */
		if (input instanceof IResourceReference) {
			Object resolved= ((IResourceReference) input).resolve();

			/* Compact Rendering */
			if (resolved instanceof ResourcePreview) {
				ResourcePreview ref= (ResourcePreview) resolved;
				if (ref.getTitle() != null)
					return ref.getTitle().getPlainText();
			}

			/* Anything Else - Delegate */
			else if (resolved != null) {
				IDomainAdapter adapter= DomainAdapterUtils.getDomainAdapter(resolved);
				if (adapter != null)
					return adapter.generateTitle(resolved);
			}
		}

		return ""; //$NON-NLS-1$
	}

	/*
	 * @see com.ibm.team.workitem.ide.ui.internal.preview.WorkItemDomainAdapter#generateJobInfo(java.lang.Object, com.ibm.team.jface.preview.IDomainAdapter.Info)
	 */
	public String generateJobInfo(Object input, Info info) {
		return Messages.ResourceReferenceDomainAdapter_LOADING_INFORMATION;
	}

	/*
	 * @see com.ibm.team.jface.preview.DomainAdapter#convertToDomainObject(java.lang.Object)
	 */
	@Override
	protected Object convertToDomainObject(Object object) {

		/* Resource Reference */
		if (object instanceof IResourceReference)
			return ((IResourceReference) object).resolve();

		return null;
	}
}