/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2006, 2016. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.calm.foundation.rcp.ui.linking;

import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.operation.IRunnableContext;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

import com.ibm.team.calm.foundation.common.ServiceProvider;
import com.ibm.team.calm.foundation.common.linking.CALMLinkTypeInformation;
import com.ibm.team.calm.foundation.common.linking.OSLCReference;
import com.ibm.team.calm.foundation.common.linking.PickerDescription;
import com.ibm.team.calm.foundation.rcp.ui.internal.CALMFoundationRCPUIPlugin;
import com.ibm.team.calm.foundation.rcp.ui.internal.linking.ItemPickerDialogDescription;
import com.ibm.team.calm.foundation.rcp.ui.internal.linking.Messages;
import com.ibm.team.process.common.IProjectAreaHandle;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.util.NLS;

public class CALMResourcePickerDialog {
	
	private final CALMLinkTypeInformation fLinkType;
	private final ServiceProviderDescriptor fServiceProviderDescriptor;
	private final Collection<ServiceProvider> fServiceProviders;
	private final ITeamRepository fRepository;
	private final IRunnableContext fRunnableContext;
	private final Shell fShell;
	private Collection<OSLCReference> fPickedReferences;
	private final String fGlobalConfigurationParameter;
	
	public CALMResourcePickerDialog(Shell shell, IRunnableContext runnableContext, CALMLinkTypeInformation linkType, IProjectAreaHandle project, Collection<ServiceProvider> serviceProviders) {
		this(shell, runnableContext, linkType, project, serviceProviders, null);
	}

	public CALMResourcePickerDialog(Shell shell, IRunnableContext runnableContext, CALMLinkTypeInformation linkType, IProjectAreaHandle project, Collection<ServiceProvider> serviceProviders, String globalConfigurationParameter) {
		fRunnableContext= runnableContext;
		fLinkType= linkType;
		fServiceProviders= serviceProviders;
		fRepository= (ITeamRepository) project.getOrigin();
		fServiceProviderDescriptor= new ServiceProviderDescriptor(linkType, project);
		fShell= shell;
		fGlobalConfigurationParameter= globalConfigurationParameter;
	}
	
	/**
	 * @deprecated use {@link #CALMResourcePickerDialog(Shell, IRunnableContext, CALMLinkTypeInformation, IProjectAreaHandle, Collection)} instead.
	 */
	public CALMResourcePickerDialog(IWorkbenchWindow window, CALMLinkTypeInformation linkType, IProjectAreaHandle project, Collection<ServiceProvider> serviceProviders) {
		this(null, window, linkType, project, serviceProviders);
	}
	/**
	 * @deprecated use {@link #CALMResourcePickerDialog(Shell, IRunnableContext, CALMLinkTypeInformation, IProjectAreaHandle, Collection)} instead.
	 */
	public CALMResourcePickerDialog(IWorkbenchWindow window, CALMLinkTypeInformation linkType, ITeamRepository repository, Collection<ServiceProvider> serviceProviders) {
		fRunnableContext= window;
		fLinkType= linkType;
		fServiceProviders= serviceProviders;
		fRepository= repository;
		fServiceProviderDescriptor= null;
		fShell= window.getShell();
		fGlobalConfigurationParameter= null;
	}
	
	public int open() {
		ServiceProviderSelectionDialog providerDialog= new ServiceProviderSelectionDialog(fLinkType, fServiceProviders, fRepository, fShell, fServiceProviderDescriptor, fGlobalConfigurationParameter);
		int result= providerDialog.open();
		if (result == Window.OK) {
			fPickedReferences= selectReferences(providerDialog);
		} else {
			fPickedReferences= Collections.<OSLCReference>emptySet();
		}
		return result;
	}
	
	public Collection<OSLCReference> getPickedReferences() {
		return fPickedReferences;
	}
	
	private Collection<OSLCReference> selectReferences(ServiceProviderSelectionDialog providerDialog) {
		if (providerDialog.getItemPickerDescription() != null) {
			ItemPickerDialogDescription pickerDialogDescription= providerDialog.getItemPickerDescription();
			if (pickerDialogDescription != null) {
				return selectResourcesWithRepositoryConnection(pickerDialogDescription, providerDialog.getLocalProject());
			}
		}
		return selectResourcesWithRemotePickerDialog(providerDialog.getOSLCPickerDescription());
	}
	
	private Collection<OSLCReference> selectResourcesWithRepositoryConnection(ItemPickerDialogDescription pickerDesc, IProjectAreaHandle project) {
		try {
			IItemPickerDialog pickerDialog= pickerDesc.getPickerDialog();
			if (pickerDialog instanceof AbstractItemPickerDialog) {
				if (fLinkType != null) {
					List<String> pickerIds= fLinkType.getPickerIds(pickerDesc.getPickerType());
					if(pickerIds != null && pickerIds.size() > 0){
						((AbstractItemPickerDialog)pickerDialog).setCalmItemType(pickerIds.get(0));
					}
				}
				if (((AbstractItemPickerDialog) pickerDialog).open(fShell, project, fRunnableContext) == Window.OK) {
					return pickerDialog.getReferences();
				}
			} else {
				IWorkbenchWindow window= fRunnableContext instanceof IWorkbenchWindow ? (IWorkbenchWindow) fRunnableContext : PlatformUI.getWorkbench().getActiveWorkbenchWindow();
				if (pickerDialog.open(project, window) == Window.OK) {
					return pickerDialog.getReferences();
				}
			}
		} catch (TeamRepositoryException ex) {
			String message= NLS.bind(Messages.CALMResourcePickerDialog_ERROR_RETRIEVE_LOCAL_DIALOG, ex.getLocalizedMessage());
			IStatus status= new Status(IStatus.ERROR, CALMFoundationRCPUIPlugin.PLUGIN_ID, IStatus.ERROR, message, ex);
			ErrorDialog.openError(fShell, Messages.CALMResourcePickerDialog_ERROR_TITLE, message, status);
		}
		return Collections.emptySet();
	}
	
	private Collection<OSLCReference> selectResourcesWithRemotePickerDialog(final PickerDescription pickerDesc) {
		OSLCPickerDialog dialog= new OSLCPickerDialog(fShell, pickerDesc);
		if (dialog.open() == Window.OK) {
			return dialog.getReferences();
		}
		return Collections.emptySet();
	}
	
	/*package*/ class ServiceProviderDescriptor {
		
		private final CALMLinkTypeInformation fCalmLinkType;
		private final IProjectAreaHandle fProjectArea;

		private ServiceProviderDescriptor(CALMLinkTypeInformation calmLinkType, IProjectAreaHandle project) {
			fCalmLinkType= calmLinkType;
			fProjectArea= project;
		}
		
		CALMLinkTypeInformation getLinkTypeInformation() {
			return fCalmLinkType;
		}
		
		@Override
		public int hashCode() {
			final int prime= 31;
			int result= 1;
			result= prime * result
					+ ((fCalmLinkType == null) ? 0 : fCalmLinkType.hashCode());
			result= prime * result
					+ ((fProjectArea == null) ? 0 : fProjectArea.hashCode());
			return result;
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			ServiceProviderDescriptor other= (ServiceProviderDescriptor)obj;
			if (fCalmLinkType == null) {
				if (other.fCalmLinkType != null)
					return false;
			} else if (!fCalmLinkType.equals(other.fCalmLinkType))
				return false;
			if (fProjectArea == null) {
				if (other.fProjectArea != null)
					return false;
			} else if (!fProjectArea.equals(other.fProjectArea))
				return false;
			return true;
		}

	}
}