/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.calm.foundation.rcp.ui.linking;

import java.io.IOException;
import java.io.StringReader;
import java.lang.reflect.Method;
import java.net.URI;
import java.net.URISyntaxException;
import java.text.MessageFormat;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Map;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.ProgressMonitorPart;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.browser.CloseWindowListener;
import org.eclipse.swt.browser.LocationEvent;
import org.eclipse.swt.browser.LocationListener;
import org.eclipse.swt.browser.OpenWindowListener;
import org.eclipse.swt.browser.ProgressEvent;
import org.eclipse.swt.browser.ProgressListener;
import org.eclipse.swt.browser.StatusTextEvent;
import org.eclipse.swt.browser.StatusTextListener;
import org.eclipse.swt.browser.TitleEvent;
import org.eclipse.swt.browser.TitleListener;
import org.eclipse.swt.browser.VisibilityWindowListener;
import org.eclipse.swt.browser.WindowEvent;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import com.ibm.team.calm.foundation.common.Size;
import com.ibm.team.calm.foundation.common.linking.OSLCReference;
import com.ibm.team.calm.foundation.common.linking.PickerDescription;
import com.ibm.team.calm.foundation.rcp.ui.internal.CALMFoundationRCPUIPlugin;
import com.ibm.team.calm.foundation.rcp.ui.internal.linking.Messages;
import com.ibm.team.repository.common.json.JSONArray;
import com.ibm.team.repository.common.json.JSONObject;
import com.ibm.team.repository.common.transport.UriUtil;


public class OSLCPickerDialog extends Dialog {
	
	private static final String PROTOCOL= "comBrowser://"; //$NON-NLS-1$
	private static final String OSLC_RESPONSE_MESSAGE= "oslc-response:"; //$NON-NLS-1$
	private static final String OSLC_RESIZE_MESSAGE= "oslc-resize:"; //$NON-NLS-1$
	
	private static final String OSLC_PARAM_WIDTH = "oslc:hintWidth";
	private static final String OSLC_PARAM_HEIGHT = "oslc:hintHeight";
	
	private final PickerDescription fPickerDescription;
	
	private Browser fBrowser;
	private Collection<OSLCReference> fResult;
	private CommunicationProtocol fCommunicationProtocol;
	private Composite fContainer;
	
	public enum CommunicationProtocol {
		WINDOW_NAME, POST_MESSAGE
	}
	
	public OSLCPickerDialog(Shell parentShell, PickerDescription pickerDescription) {
		super(parentShell);

		setShellStyle(getShellStyle() | SWT.RESIZE);
		
		fPickerDescription= pickerDescription;
		fCommunicationProtocol= CommunicationProtocol.POST_MESSAGE;
	}
	
	public void setCommunicationProtocol(CommunicationProtocol protocol) {
		fCommunicationProtocol= protocol;
	}
	
	public Collection<OSLCReference> getReferences() {
		return fResult;
	}
	
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setText(fPickerDescription.getTitle());
	}
	
	@Override
	protected Control createButtonBar(Composite parent) {
		// no buttons
		return null;
	}

	@Override
	protected Control createDialogArea(Composite parent) {
		
		fContainer= (Composite) super.createDialogArea(parent);
		
		GridData gd= (GridData) fContainer.getLayoutData();
		gd.widthHint= fPickerDescription.getWidthHintInPixels(500);
		gd.heightHint= fPickerDescription.getHeightHintInPixels(200);
		
		GridLayout layout = (GridLayout) fContainer.getLayout();
		layout.marginHeight= 0;
		layout.marginWidth= 0;
		
		String protocol;
		if (fCommunicationProtocol == CommunicationProtocol.POST_MESSAGE) {
			protocol= "oslc".equals(fPickerDescription.getResultNamespace()) ? "#oslc-core-postMessage-1.0" : "#oslc-postMessage-1.0"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		} else {
			protocol= "oslc".equals(fPickerDescription.getResultNamespace()) ? "#oslc-core-windowName-1.0" : "#oslc-windowName-1.0"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}

		final String url= fPickerDescription.getPickerURL() + protocol;
		final String returnURL= url + "-return-location"; //$NON-NLS-1$

		final Composite composite= new Composite(fContainer, SWT.NONE);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		composite.setLayout(new StackLayout());
		
		fBrowser= new Browser(composite, SWT.NONE);
		
		final Composite progress= new Composite(composite, SWT.NONE);
		progress.setLayout(new GridLayout());

		final ProgressMonitorPart monitor= new ProgressMonitorPart(progress, new GridLayout());
		monitor.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, true));
		((StackLayout)composite.getLayout()).topControl= progress;
		monitor.beginTask(Messages.OSLCPickerDialog_PROGRESS_TASK, 5); // progress seems to be reported 2 times, but 5 looks better
		
		fBrowser.addOpenWindowListener(new OpenWindowListener() {
			public void open(WindowEvent event) {
				
				Shell shell= new Shell (fBrowser.getShell());
				shell.setLayout(new FillLayout());
				final Browser logInBrowser = new Browser(shell, SWT.NONE);
				event.browser = logInBrowser;
				
				logInBrowser.addVisibilityWindowListener(new VisibilityWindowListener() {
					
					@Override
					public void show(WindowEvent event) {
						Shell shell= logInBrowser.getShell();
						if (event.location != null) {
							shell.setLocation(event.location);
						}
						
						if (event.size != null) {
							shell.setSize(650, 500);
						}
						shell.open();
					}
					
					@Override
					public void hide(WindowEvent event) {
					}
				});
				
				logInBrowser.addCloseWindowListener(new CloseWindowListener() {
					public void close(WindowEvent event) {
						Browser browser = (Browser) event.widget;
						Shell shell = browser.getShell();
						shell.close();
					}
				});
			}
		});

		fBrowser.addKeyListener(new KeyListener() {
			public void keyPressed(KeyEvent evt) {
				if (evt.character == 0x1B) // ESC 
					close();
			}
			public void keyReleased(KeyEvent evt) {}
		});
		fBrowser.addProgressListener(new ProgressListener() {
			public void completed(ProgressEvent event) {
				if (!progress.isDisposed()) {
					monitor.done();
					progress.dispose();
				}
				fBrowser.setVisible(true);
				((StackLayout)composite.getLayout()).topControl= fBrowser;
			}
			public void changed(ProgressEvent evt) {
				if (!monitor.isDisposed())
					monitor.worked(evt.current);
			}
		});
		
		if (fCommunicationProtocol == CommunicationProtocol.POST_MESSAGE) {
			class Listener implements StatusTextListener, TitleListener {
				private String fLastMessage= null;
				
				public void changed(TitleEvent event) {
					try {
						String result= (String) fBrowser.evaluate("return window.oslcResult;");
						textChanged(result);
					} catch (Exception e) {
						// no luck, ignore all exceptions (NoSuchMethod, CastException...)
					}
				}
				public void changed(StatusTextEvent event) {
					textChanged(event.text);
				}
				private void textChanged(String text) {
					if (!text.equals(fLastMessage) && text.startsWith(PROTOCOL)) {
						performMessagePosted(text.substring(PROTOCOL.length()));
					}
					fLastMessage= text;
				}
			}
			
			Listener listener= new Listener();
			fBrowser.addStatusTextListener(listener);
			fBrowser.addTitleListener(listener); // also listen for title changes: workound for Eclipse 3.61 problem: 159482: OSLC dialogs not closing on 3.6 Eclipse client
		}
		
		fBrowser.addLocationListener(new LocationListener() {

			public void changed(LocationEvent event) {
				if (isEqualURL(event.location, fPickerDescription.getPickerURL())) {
					fBrowser.setUrl(url);
				} else if (isEqualURL(event.location, url)) {
					fBrowser.execute("document.getElementsByTagName('body')[0].style.overflow='hidden'"); //$NON-NLS-1$
					fBrowser.execute("window.postMessage= function(e) { var r= '" + PROTOCOL + "' + e; window.status= r; if (window.status != r) { window.oslcResult= r; window.document.title= r; } }"); //$NON-NLS-1$
					fBrowser.execute("window.addEventListener('message', function(e) { var r= '" + PROTOCOL + "' + e.data; window.status= r; if (window.status != r) { window.oslcResult= r; window.document.title= r; } })"); //$NON-NLS-1$				
				}
				if (fCommunicationProtocol == CommunicationProtocol.WINDOW_NAME) {
					if (isEqualURL(event.location, url)) {
						setWindowName(returnURL);
					}
					if (isEqualURL(event.location, returnURL)) {
						String result= getWindowName();
						if (result != null) {
							if (processResult(result)) {
								event.doit= false;
							}
						}
					}
				}
			}
			
			public void changing(LocationEvent event) {
				if (fCommunicationProtocol == CommunicationProtocol.WINDOW_NAME) {
					if (isEqualURL(event.location, returnURL)) {
						String result= getWindowName();
						if (result != null) {
							if (processResult(result)) {
								event.doit= false;
							}
						}
					}
				}
			}
			
			private void setWindowName(String newLocation) {
				String command= "window.name= '" + newLocation + "';"; //$NON-NLS-1$  //$NON-NLS-2$
				fBrowser.execute(command);
			}
			
			private String getWindowName() {
				StatusTextListenerImpl listener= new StatusTextListenerImpl();
				fBrowser.addStatusTextListener(listener);
				try {
					String command= "window.status= '" + PROTOCOL + "' + window.name;"; //$NON-NLS-1$  //$NON-NLS-2$
					boolean scriptExecuted= fBrowser.execute(command);
					if (!scriptExecuted || listener.result == null) {
						return getWindowNameWithEvaluate();
					}
					return listener.result;
				} finally {
					fBrowser.removeStatusTextListener(listener);
				}
			}
			
			private String getWindowNameWithEvaluate() {
				try {
					Method evaluateMethod= fBrowser.getClass().getMethod("evaluate", String.class); //$NON-NLS-1$
					return (String) evaluateMethod.invoke(fBrowser, "return window.name;"); //$NON-NLS-1$
				} catch (Exception e) {
					// no luck, ignore all exceptions (NoSuchMethod, CastException...)
				}
				return null;
			}

			class StatusTextListenerImpl implements StatusTextListener {
				public String result;
				public void changed(StatusTextEvent event) {
					String text= event.text;
					this.result= text.startsWith(PROTOCOL) ? text.substring(PROTOCOL.length()) : null;
				}
			}
		});
		
		fBrowser.setUrl(url);
		
		applyDialogFont(fContainer);
		return fContainer;
	}
	
	protected void performMessagePosted(String text) {
		if (text.startsWith(OSLC_RESPONSE_MESSAGE)) {
			processResult(text.substring(OSLC_RESPONSE_MESSAGE.length()));
		}
		if (text.startsWith(OSLC_RESIZE_MESSAGE)) {
			processResize(text.substring(OSLC_RESIZE_MESSAGE.length()));
		}
	}

	protected void processResize(String sizeString) {
		try {
			if (fContainer == null)
				return;
			
			JSONObject newSizeObj= JSONObject.parse(new StringReader(sizeString));
			Object widthObj= newSizeObj.get(OSLC_PARAM_WIDTH);
			Object heightObj= newSizeObj.get(OSLC_PARAM_HEIGHT);
			String widthStr= (widthObj instanceof String) ? (String) widthObj : "-1px"; //$NON-NLS-1$
			String heightStr= (heightObj instanceof String) ? (String) heightObj : "-1px"; //$NON-NLS-1$
			
			boolean hasChanged= false;
			Size newSize= Size.parse(widthStr, heightStr, new Size(-1, -1, Size.SizeUnit.px));
			GridData gridData= ((GridData) fContainer.getLayoutData());
			int newWidth= newSize.getWidth(Size.SizeUnit.px);
			int newHeight= newSize.getHeight(Size.SizeUnit.px);
			if (newWidth != -1 && gridData.widthHint != newWidth) {
				gridData.widthHint= newWidth;
				hasChanged= true;
			}
			if (newHeight != -1 && gridData.heightHint != newHeight) {
				gridData.heightHint= newHeight;
				hasChanged= true;
			}
			if (hasChanged) {
				initializeBounds();
			}
			
		} catch (IOException e) {
			// ignore malformed JSON
		}
	}

	protected boolean processResult(String data) {
		if (data.length() != 0) {
			fResult= parseReferencesFromJson(data);
			setReturnCode(Window.OK);
			getParentShell().getDisplay().asyncExec(new Runnable() { //See defect 107026
				public void run() {
					close();
				}
			});
			return true;
		}
		return false;
	}

	private Collection<OSLCReference> parseReferencesFromJson(String data) {
		String namespace= fPickerDescription.getResultNamespace();
		String resultsKey= namespace + ":results"; //$NON-NLS-1$
		
		JSONObject jsResult= null;
		try {
			jsResult= JSONObject.parse(new StringReader(data));
		} catch (IOException ex) {
			String message= MessageFormat.format(Messages.OSLCPickerDialog_ERROR_DIALOG_RESULT, data);
			Status status= new Status(IStatus.ERROR, CALMFoundationRCPUIPlugin.PLUGIN_ID, IStatus.ERROR, message, ex);
			CALMFoundationRCPUIPlugin.getDefault().log(status);
		}
		if (jsResult != null) {
			String labelKey= namespace + ":label";  //$NON-NLS-1$
			String resourceKey= "rdf:resource"; //$NON-NLS-1$
			
			Object object= jsResult.get(resultsKey);
			if (!(object instanceof JSONArray) && "oslc_rm".equals(namespace)) { //$NON-NLS-1$
				// see bug 13645
				object= jsResult.get("http://open-services.net/xmlns/rm/1.0/web/results"); //$NON-NLS-1$
				resourceKey= "http://www.w3.org/1999/02/22-rdf-syntax-ns#resource"; //$NON-NLS-1$
				labelKey= "http://www.w3.org/2000/01/rdf-schema#label"; //$NON-NLS-1$
			}
			if (object instanceof JSONArray) {
				JSONArray jsElements= (JSONArray) object;
				Collection<OSLCReference> result= new HashSet<OSLCReference>();
				for (Object cur : jsElements) {
					if (cur instanceof JSONObject) {
						OSLCReference curRef= parseReferenceFromJson((JSONObject)cur, labelKey, resourceKey);
						if (curRef != null) {
							result.add(curRef);
						}
					}
				}
				return result;
			}
		}
		return Collections.emptySet();
	}

	private OSLCReference parseReferenceFromJson(JSONObject jsObject, String labelKey, String resourceKey) {
		Object url= jsObject.get(resourceKey);
		Object label= jsObject.get(labelKey);
		if (url instanceof String && label instanceof String) {
			try {
				URI uri= new URI((String)url);
				return new OSLCReference(uri, (String)label);
			} catch (URISyntaxException ex) {
				Status status= new Status(IStatus.ERROR, CALMFoundationRCPUIPlugin.PLUGIN_ID, IStatus.ERROR, Messages.OSLCPickerDialog_ERROR_INVALID_URL, ex);
				CALMFoundationRCPUIPlugin.getDefault().log(status);
			}
		} else {
			String message= MessageFormat.format(Messages.OSLCPickerDialog_ERROR_CANNOT_CREATE_REFERENCE_FROM_JSON, url);
			Status status= new Status(IStatus.ERROR, CALMFoundationRCPUIPlugin.PLUGIN_ID, IStatus.ERROR, message, null);
			CALMFoundationRCPUIPlugin.getDefault().log(status);
		}
		return null;
	}	
	
	private static URI getURI(String uriString) throws URISyntaxException {
		URI uri= new URI(uriString);
		int port= uri.getPort();
		if (port == -1) {
			port= "https".equalsIgnoreCase(uri.getScheme()) ? 443 : 80; //$NON-NLS-1$
		}
		return new URI(uri.getScheme(), uri.getUserInfo(), uri.getHost(), port, uri.getPath(), "", uri.getFragment()); //$NON-NLS-1$
	}
	
	/**
	 * Workaround for 166713: Authentication redirect changes order of URL query parameters
	 */
	private static boolean isEqualQuery(String q1, String q2) {
		if (q1 == null || q2 == null) {
			return q1 == q2;
		}
		Map<String, String[]> m1= UriUtil.parseQueryParameters(q1);
		Map<String, String[]> m2= UriUtil.parseQueryParameters(q2);
		if (m1.size() != m2.size()) {
			return false;
		}
		for (Map.Entry<String, String[]> entry1: m1.entrySet()) {
			String[] values1= entry1.getValue();
			String[] values2= m2.get(entry1.getKey());
			if (values1 == null || values2 == null) {
				return values1 == values2;
			}
			
			if (values1.length != values2.length) {
				return false;
			}
			Arrays.sort(values1);
			Arrays.sort(values2);
			if (!Arrays.equals(values1, values2)) {
				return false;
			}
			
		}
		return true;
	}
	
	private static boolean isEqualURL(String url1, String url2) {
		if (url1.equals(url2)) {
			return true;
		}
		try {
			URI uri1= getURI(url1);
			URI uri2= getURI(url2);
			return uri1.equals(uri2) && isEqualQuery(uri1.getQuery(), uri2.getQuery());
		} catch (URISyntaxException ignored) {}

		return false;
	}
}
