/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2010. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.calm.foundation.rcp.ui.preview;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.Control;

import com.ibm.team.calm.foundation.client.preview.ResourceReferenceResolver;
import com.ibm.team.calm.foundation.common.preview.IResourceReferenceResolver;
import com.ibm.team.calm.foundation.common.preview.IResourceReferenceResolver.IResourceReferenceListener;
import com.ibm.team.links.common.IReference;
import com.ibm.team.repository.client.ITeamRepository;


/**
 * A structured content provider that resolves instances of <code>IReference</code> within a 
 * background operation. It show information available at a reference immediately and replaces this 
 * information once the reference is resolved. The resolution is provided by 
 * <code>IResourceReferenceResolver</code>.
 * 
 * @author Jan Wloka
 * @since 3.0
 */
public class ResourceReferenceContentProvider implements IStructuredContentProvider {

	private final IResourceReferenceResolver fResolver;
	private final ITeamRepository fTeamRepository;
	private IResourceReferenceListener fResolveListener;

	public ResourceReferenceContentProvider(ITeamRepository teamRepository) throws IllegalArgumentException {
		if (teamRepository == null) {
			throw new IllegalArgumentException();
		}
		fTeamRepository= teamRepository;
		fResolver= new ResourceReferenceResolver();
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
	 */
	public Object[] getElements(Object inputElement) {
		if (inputElement instanceof Object[]) {
			return triggerElementResolution((Object[])inputElement);
		}
		return new Object[0];
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
	 */
	public void inputChanged(final Viewer viewer, Object oldInput, Object newInput) {
		if (viewer instanceof StructuredViewer) {
			fResolveListener= new IResourceReferenceListener() {
				public void referencesUpdated(final List<IReference> references) {
					Control control= viewer.getControl();
					if (control != null && !control.isDisposed() && references != null) {
						control.getDisplay().asyncExec(new Runnable() {
							public void run() {
								((StructuredViewer)viewer).update(references.toArray(), null); // properties are unknown
							}
						});
					}
				}
			};
			fResolver.addReferencesListener(fResolveListener);
		} else {
			fResolver.removeReferencesListener(fResolveListener);
			fResolveListener= null;
		}
	}

	/**
	 * Default implementation does nothing.
	 * 
	 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
	 */
	public void dispose() {}

	
	protected Object triggerElementResolution(Object element) {
		if (element instanceof IReference) { 
			IReference resolvable= fResolver.makeResolvable((IReference)element, fTeamRepository);
			fResolver.resolveInBackground(Arrays.asList(resolvable));
			
			return resolvable;
		}	
		return element;
	}
	
	protected Object[] triggerElementResolution(Object[] elements) {
		if (elements.length > 0 && elements[0] instanceof IReference) { 
			List<IReference> resolvable= new ArrayList<IReference>();
			for (Object cur : elements) {
				if (cur instanceof IReference) {
					resolvable.add(fResolver.makeResolvable((IReference)cur, fTeamRepository));
				}
			}
			fResolver.resolveInBackground(resolvable);
			
			return resolvable.toArray();
		}	
		return elements;
	}
}
