/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2011. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.calm.foundation.rcp.ui.preview;

import java.net.URI;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ViewerLabel;
import org.eclipse.swt.graphics.Image;

import com.ibm.team.calm.foundation.client.preview.IResourceReference;
import com.ibm.team.calm.foundation.common.preview.ResourcePreview;
import com.ibm.team.calm.foundation.rcp.ui.internal.ImagePool;
import com.ibm.team.foundation.common.text.XMLString;
import com.ibm.team.jface.JazzResources;
import com.ibm.team.jface.labelProviders.ElementRemovedNotifierImpl;
import com.ibm.team.jface.labelProviders.StandardLabelProvider;
import com.ibm.team.jface.labelProviders.TypeLabelProvider;
import com.ibm.team.process.common.IProjectAreaHandle;


/**
 * A standard label provider contribution for {@link IResourceReference}.
 *  
 * @noextend This class is not intended to be subclassed by clients.
 * @since 3.0
 */
public class ResourceReferenceLabelProvider extends TypeLabelProvider {

	private StandardLabelProvider fStandardLabelProvider;
	
	/**
	 * Creates a resource reference label provider.
	 */
	public ResourceReferenceLabelProvider() {
		fStandardLabelProvider= new StandardLabelProvider(new ElementRemovedNotifierImpl());
	}
	
	/**
	 * Updates the label if the given element is of type {@link IResourceReference}.
	 * The label is updated to the text and icon provided for the resource the reference
	 * is pointing to. The reference's comment and a default icon is used if the reference is not
	 * resolved.
	 * 
	 * @param label - the label to be updated, may be null
	 * @param element - the {@link IResourceReference}, may be null
	 */
	@Override
	public void updateLabel(ViewerLabel label, Object element) {
		if (label != null && element instanceof IResourceReference) {
			IResourceReference ref= (IResourceReference)element;
			Object resource= ref.resolve();
			if (resource != null) {
				if (resource instanceof ResourcePreview) {
					ResourcePreview details= (ResourcePreview)resource;
					updateText(label, details);
					updateImage(label, details);
				} else {
// FIXME(CLM) Remove check if process can deal with null origin.
					if (!(resource instanceof IProjectAreaHandle) || ((IProjectAreaHandle)resource).getOrigin() != null) {
						fStandardLabelProvider.updateLabel(label, resource);
					}
				}
			} else {
				updateText(label, ref.getURI(), ref.getComment());
				updateImageWithDefault(label);
			}
		}
	}

	/**
	 * Disposes acquired resources.
	 */
	@Override
	public void dispose() {
		if (fStandardLabelProvider != null) {
			fStandardLabelProvider.dispose();
			fStandardLabelProvider= null;
		}
		super.dispose();
	}
	
	private void updateText(ViewerLabel label, ResourcePreview resource) {
		XMLString title= resource.getTitle();
		if (title != null) {
			String text= title.getPlainText();
			if (isValidString(text)) {
				label.setText(text);
			}
		}
	}
	
	private void updateText(ViewerLabel label, URI uri, String comment) {
		String text= comment;
		if (uri != null && !isValidString(text)) {
			text= uri.toString();
		}
		if (isValidString(text)) {
			label.setText(text);
		}
 	}

	private void updateImage(ViewerLabel label, ResourcePreview resource) {
		Image image= null;
		try {
			URI icon= resource.getIconURI();
			if (icon != null) {
				ImageDescriptor desc= ImageDescriptor.createFromURL(icon.toURL());
				image= getResources().createImage(desc);
			}
		} catch (Exception ignored) {}
		
		if (image != null) {
			label.setImage(image);
		}
	}
	
	private void updateImageWithDefault(ViewerLabel label) {
		Image defaultImage= JazzResources.getImageWithDefault(getResources(), ImagePool.RELATED_ARTIFACT_OBJ);
		label.setImage(defaultImage);
	}

	private boolean isValidString(String str) {
		return str != null && str.length() > 0;
	}
}
